import datetime
import httplib
import logging
import re

from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail
from django.http import Http404, HttpResponseNotAllowed, HttpResponseBadRequest, HttpResponse
from django.shortcuts import render_to_response
from django.template import Context, RequestContext, loader

from emailthis.forms import EmailEventForm
from emailthis.models import EmailEvent
from emailthis.util import render_to_json, get_subject, clean_errors

def get_email_form(request, content_type_id, object_id):
    """
    returns an html form suitable for the specified object.
    """

    try:
        content_type=ContentType.objects.get(pk=content_type_id)
    except ObjectDoesNotExist:
        raise Http404("no such content type")
    try:
        item=content_type.get_object_for_this_type(pk=object_id)

    except ObjectDoesNotExist:
        raise Http404("no such object id")
    subject=get_subject(item)
    user=request.user if request.user.is_authenticated() else None
    initial={'subject' : subject}
    if user and user.email:
        initial['email_from']=user.email
        
    d={'form' : EmailEventForm(initial=initial),
       'item' : item,
       'content_type' : content_type}
    return render_to_response('emailthis/form.html', d, context_instance=RequestContext(request))


def process_email_form(request, content_type_id=None, object_id=None):
    """
    submission processor for email form.
    """
    if not (request.method=='POST' and request.is_ajax()):
        return HttpResponse('Method Not Allowed', status=httplib.METHOD_NOT_ALLOWED, mimetype='text/plain')
    if content_type_id is None:
        content_type_id=request.POST.get('content_type')
    try:
        content_type=ContentType.objects.get(pk=int(content_type_id))
    except (ObjectDoesNotExist, ValueError, TypeError):
        return HttpResponseBadRequest("invalid content type", mimetype='text/plain')
    if object_id is None:
        object_id=request.POST.get('object_id')
    try:
        item=content_type.get_object_for_this_type(pk=int(object_id))
    except (ObjectDoesNotExist, ValueError, TypeError):
        return HttpResponseBadRequest("no such object id")

    form=EmailEventForm(request.POST)
    if not form.is_valid():
        logging.debug("form is invalid.  Errors are: %s", form.errors)
        logging.debug("type of errors: %s", type(form.errors))
        return render_to_json(clean_errors(form.errors), status=httplib.BAD_REQUEST)
    
    cleaned=form.cleaned_data
    message_template=loader.get_template('emailthis/email_message.txt')
    site=Site.objects.get_current()
    user=request.user if request.user.is_authenticated() else None
    url=item.get_absolute_url()
    if not re.match('https?://', url):
        url='http://%s%s' % (site.domain, url)
    message_context=Context(dict(
        email_from=cleaned['email_from'],
        subject=cleaned['subject'],
        message=cleaned['message'],
        site=site,
        site_url='http://%s/' % site.domain,
        url=url,
        item=item,
        user=user,
        ))
    message=message_template.render(message_context)
    
    recipients=cleaned['email_to'].split(',')
    from_address=cleaned['email_from']
    send_mail(cleaned['subject'], message, from_address, recipients, fail_silently=False)
    event=form.save(commit=False)
    event.remote_ip=request.META['REMOTE_ADDR']
    event.content_type_id=content_type.pk
    event.object_id=item.pk
    event.mailed_by=user
    event.save()
    return HttpResponse("OK", mimetype="text/plain")

try:
    from djangologging.decorators import suppress_logging_output
except ImportError:
    pass
else:
    get_email_form=suppress_logging_output(get_email_form)
    process_email_form=suppress_logging_output(process_email_form)
    



"""

A Django implementation of email-to-a-friend
that works with unregistered or registered users.

"""

import datetime

from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes.models import ContentType
import django.db

from emailthis.models import EmailEvent

def _safe_get_object(ctype, pk):
    try:
        return ctype.get_object_for_this_type(pk=pk)
    except ObjectDoesNotExist:
        return None

def get_most_emailed(numdays=7, limit=10):
    """
    Returns the most emailed <limit> things of the last <numdays> days as a list
    of (thing, count) tuples.
    """
    sql="""
    SELECT count(e.*) as cnt, e.content_type_id, e.object_id
    FROM emailthis_emailevent e
    WHERE happened_at >= %%s
    GROUP BY e.content_type_id, e.object_id
    ORDER BY cnt DESC
    LIMIT %d

    """ % limit
    when=datetime.datetime.now() - datetime.timedelta(days=numdays)
    c=django.db.connection.cursor()
    c.execute(sql, (when,))
    res=c.fetchall()
    ctypes=dict((x.pk, x) for x in ContentType.objects.filter(pk__in=set(r[1] for r in res)))
    return [(x, y) for x, y in ((_safe_get_object(ctypes[ct], oid), cnt) for cnt, ct, oid in res) if x]



import re

from django import forms
try:
    from django.core.validators import email_re    
except ImportError:
    # django 1.1
    from django.forms.fields import email_re

from emailthis.models import EmailEvent

class MultiEmailField(forms.CharField):
    
    def clean(self, value):
        if not value:
            raise forms.ValidationError('Enter at least one e-mail address.')
        emails = set(re.split('\s*,?\s*', value))
        for email in emails:
            if not email_re.match(email):
                raise forms.ValidationError('%s is not a valid e-mail address.' % email)
        return ','.join(list(emails))


class EmailEventForm(forms.ModelForm):
    email_from=forms.EmailField(label="Your Email", help_text="required", required=True)
    email_to=MultiEmailField(label="Addressees",
                             required=True,
                             help_text="required, please separate multiple addresses with commas")
    subject=forms.CharField(label="Subject", required=True)
    message=forms.CharField(label="Personal Message", widget=forms.Textarea, required=False)


    class Meta:
        model=EmailEvent
        exclude=('mailed_by', 'happened_at', 'content_type', 'object_id', 'remote_ip')

                                                        



from django.conf.urls.defaults import patterns, url

urlpatterns=patterns(
    'emailthis.views',
    url(r'^(\d+)/(\d+)/$', 'get_email_form', name="emailthis_form"),
    url(r'^submit/(\d+)/(\d+)/$', 'process_email_form', name="emailthis_submit")
    )
                     



try:
    from setuptools import setup
except ImportError:
    from distutils import setup

description="""
a simple email-to-a-friend application that generates an email
form, handles its submission, persists the submission data to a
database, and sends a templated email.  Both anonymous and registered
users can send email.
"""

long_description=description + """

This app started out as a quasi-fork of Jeff Croft's "mailfriend" app
(http://code.google.com/p/django-mailfriend/), but the latter only
supported registered users; this supports users of both
the registered and unregistered varieties.

Form submission is here done solely via ajax; form loading is intended
to be handled similarly.

"""

version="0.2.1"

setup(author="Jacob Smullyan",
      author_email='jsmullyan@gmail.com',
      description=description,
      long_description=long_description,
      license="BSD",
      platforms='OS Independent',
      name="django-emailthis",
      url="http://code.google.com/p/django-emailthis/",
      classifiers=["Development Status :: 3 - Alpha",
                   "Environment :: Web Environment",
                   "Framework :: Django",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: BSD License",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                   'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      version=version,
      keywords="django email",
      packages=("emailthis",),
      package_dir={'' : '.'}
      )





import datetime

from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType

class EmailEvent(models.Model):
    content_type=models.ForeignKey(ContentType)
    object_id=models.IntegerField()
    content_object=generic.GenericForeignKey()
    
    mailed_by=models.ForeignKey(User, null=True, blank=True)
    email_from=models.EmailField("Your Email", help_text="required")
    email_to=models.CharField(max_length=300)    
    subject=models.CharField("Subject", max_length=120)
    message=models.TextField("Personal Message", default='', blank=True)
    
    remote_ip=models.IPAddressField()
    happened_at=models.DateTimeField(editable=False, default=datetime.datetime.now)

    def __unicode__(self):
        return "%s emailed by %s to %s" % (self.content_object,
                                           self.email_from,
                                           self.email_to)
                                           



import httplib

try:
    # prefer externally maintained version
    import simplejson
except ImportError:
    # use bundled version
    from django.utils import simplejson


from django.conf import settings
from django.http import HttpResponse

def render_to_json(data, status=httplib.OK):
    return HttpResponse(simplejson.dumps(data),
                        mimetype='application/json',
                        status=status)

def clean_errors(djerrors):
    d={}
    for k, v in djerrors.iteritems():
        d[unicode(k)]=[unicode(x) for x in v]
    return d
    
try:
    get_subject=settings.EMAILTHIS_SUBJECT_GETTER
except AttributeError:
    def get_subject(item):
        return str(item)

